#include "Graph.h"
#include "GraphAlgo.h"


void GraphNode::add_neigbor(GraphNode* neighboor)
{
	neighbors.insert(neighboor);
}

void GraphNode::remove_neighbor(GraphNode* neighboor)
{
	neighbors.erase(neighboor);
}

GraphNode::GraphNode()
{
}

GraphNode::GraphNode(const string& aname)
{
	name = aname;
}

const string GraphNode::get_name() const
{
	return name;
}

const bool& GraphNode::operator==(const GraphNode& r)
{
	return name == r.name;
}

void Graph::add_node(GraphNode* GraphNode)
{
	if (taken_names[GraphNode->name]) {
		throw Graph_exception();
	}
	else {
		taken_names[GraphNode->name] = true;
		nodes.insert(GraphNode);
	}
}

void Graph::remove_node(GraphNode* GraphNode)
{
	nodes.erase(GraphNode);
	for (auto it = nodes.begin(); it != nodes.end(); it++) {
		(*it)->remove_neighbor(GraphNode);
	}

}

void Graph::add_edge(GraphNode* begin, GraphNode* end)
{
	if (nodes.find(begin) == nodes.end())
		return;
	if (nodes.find(end) == nodes.end())
		return;
	begin->add_neigbor(end);
	end->add_neigbor(begin);
}

void Graph::remove_edge(GraphNode* begin, GraphNode* end)
{
	if (nodes.find(begin) == nodes.end())
		return;
	if (nodes.find(end) == nodes.end())
		return;
	begin->remove_neighbor(end);
	end->remove_neighbor(begin);
}



Graph Graph::read_file(const string &file_name)
{
	Graph graph = Graph();
	ifstream input;
	input.open(file_name);
	if (input.is_open()) {
		string line;
		getline(input, line);

		while (getline(input, line)) {
			string fir = "", sec = "";
			int splitter = 0;
			while (line[splitter] != '\t')
			{
				fir += line[splitter];
				splitter++;
			}
			splitter++;
			while (line[splitter])
			{
				sec += line[splitter];
				splitter++;
			}

			graph.saved[fir] = graph.taken_names[fir] ? graph.saved[fir] : GraphNode(fir);
			graph.saved[sec] = graph.taken_names[sec] ? graph.saved[sec] : GraphNode(sec);

			try {
				graph.add_node(&graph.saved[fir]);
			}
			catch (Graph_exception ex) {}
			try {
				graph.add_node(&graph.saved[sec]);
			}
			catch (Graph_exception ex) {}

			graph.add_edge(&graph.saved[fir], &graph.saved[sec]);
		}
	}
	input.close();

	return graph;
}

void Graph::write_graph(vector<vector<node_iterator>> ind)
{
	int size = ind.size();
	for (int i = 0; i < size; i++) {

		string c_name = "graph";
		c_name += (i + 49);
		c_name += ".txt";

		ofstream out(c_name);
		for (int j = 0; j < ind[i].size(); j++) {
			auto& cur = *ind[i][j];
			auto it = cur->neighbors.begin();
			while (it != cur->neighbors.end()) {
				auto neighb = *it;
				out << cur->name << '\t' << neighb->name << '\n';
				it++;
			}
		}
		out.close();
	}
}

void Graph::find_independent_graphs()
{
	int size = nodes.size();
	vector <vector <node_iterator> > independent;
	set<GraphNode*> working_nodes = nodes;


	for (int i = 0; i < size; i++) {

		set<GraphNode*>::iterator it = working_nodes.begin(), compared = working_nodes.begin();
		vector<node_iterator> tmp_nodes;
		tmp_nodes.push_back(nodes.find(*it));

		BFS fs(this);
		compared++;
		while (compared != working_nodes.end()) {

			if (fs.connected(*it, *compared))
				tmp_nodes.push_back(nodes.find(*compared));
			compared++;

		}

		independent.push_back(tmp_nodes);

		for (int i = 0; i < tmp_nodes.size(); i++) {
			working_nodes.erase(working_nodes.find(*tmp_nodes[i]));
			size--;
		}
	}

	write_graph(independent);
}